@@ -155,6 +155,15 @@ int compat_fclose(FILE *stream)
 	return fclose(stream);
 }
 
+void compat_binmode(int fd, int enable)
+{
+#if   defined(HAVE__SETMODE)
+	_setmode(fd, enable ? _O_BINARY : _O_TEXT);
+#elif defined(HAVE_SETMODE)
+	setmode(fd, enable ? O_BINARY : O_TEXT);
+#endif
+}
+
 #ifndef WINDOWS_UWP
 
 /*
@@ -441,7 +450,12 @@ size_t unintr_write(int fd, void const *buffer, size_t bytes)
 		{
 			bytes   -= part;
 			written += part;
-		} else if(errno != EINTR)
+		} else if(errno != EINTR && errno != EAGAIN
+#ifndef __KLIBC__
+			// OS/2 is funny with POSIX.
+			&& errno != EWOULDBLOCK
+#endif
+		)
 			break;
 	}
 	return written;
@@ -456,11 +470,16 @@ size_t unintr_read(int fd, void *buffer, size_t bytes)
 	{
 		errno = 0;
 		ssize_t part = read(fd, (char*)buffer+got, bytes);
-		if(part >= 0)
+		if(part > 0) // == 0 is end of file
 		{
 			bytes -= part;
 			got   += part;
-		} else if(errno != EINTR)
+		} else if(errno != EINTR && errno != EAGAIN
+#ifndef __KLIBC__
+			// OS/2 is funny with POSIX.
+			&& errno != EWOULDBLOCK
+#endif
+		)
 			break;
 	}
 	return got;
